#!/usr/bin/python3
#coding -*-coding:Utf-8 -*

# Import
import glob, os, sys # basic files tools
from matplotlib.pyplot import *


from definitions import *
import subprocess

#########################################################
# BODY OF THE PROGRAM
#########################################################




def runFit(s_filename, s_counter, s_outfile): 
    subprocess.call(["./spfit", s_filename + '.lin']) # run spfit
    
    with open(s_filename + '.fit', 'r') as fit_file:
        for line in reversed(list(fit_file)):
            if ('END OF ITERATION' in line):
                rms1 = line.split()[8] 
                rms2 = line.split()[9]                    
            if (' 30000 ' in line):
                ctCf = line.split()[2]
                ctCf = ctCf.split('(')[0]
                ctCf = float(ctCf)                        
            if (' 20000 ' in line):
                ctBf = line.split()[2]
                ctBf = ctBf.split('(')[0] 
                ctBf = float(ctBf)            
            if (' 10000 ' in line):
                ctAf = line.split()[2]
                ctAf = ctAf.split('(')[0]
                ctAf = float(ctAf)       
                break
        if (ctAf >= ctBf >= ctCf and float(rms2) > 0.0 and float(rms1)-float(rms2) < 0.01 and ctCf>0):
            rms_list.append(float(rms1))
            index_list.append(s_counter)
            print('\n---------------------------------------------------------')
            print('GOOD FIT'.center(55))
            print('A = ', ctAf, '   B = ', ctBf,'   C = ', ctCf, ' (MHz)')
            print('Final rms: ', rms1)
            print('---------------------------------------------------------\n')
            
            s_outfile.write('FIT %2s: GOOD FIT\n' %(s_counter))
            s_outfile.write('A = %2s,  B = %2s,  C = %2s (MHz)\n' %(ctAf,ctBf,ctCf))
            s_outfile.write('Final rms: %2s\n\n' %(rms1))
        else:
            print('\n---------------------------------------------------------')
            print('BAD FIT'.center(55))
            print('A = ', ctAf, '   B = ', ctBf,'   C = ', ctCf, ' (MHz)')
            print('Final rms: ', rms1, rms2)
            print('---------------------------------------------------------\n')
            
            s_outfile.write('FIT %2s: BAD FIT\n' %(s_counter))
            s_outfile.write('A = %2s,  B = %2s,  C = %2s (MHz)\n' %(ctAf,ctBf,ctCf))
            s_outfile.write('Final rms: %2s %2s \n\n'%(rms1, rms2))
    
    fit_file.close()    


#--------------------------------------------------------------------
# Clean the work directory
#--------------------------------------------------------------------
cleanPath()


#--------------------------------------------------------------------
# Selections rules
#--------------------------------------------------------------------

a_rule = np.array([1,0,1])
#bc_rule = np.array([ [1,1,1], [1,1,-1], [1,-1,1], [-1,1,1], [-1,1,-1], [-1,-1,1], [1,1,0], [-1, -1, 0] ]) #Need to code delta J = -1 as we don't know which ladder is the lowest Ka
bc_rule = np.array([ [1,1,1], [1,1,-1], [1,-1,1], [-1,-1,-1], [-1,-1,1], [-1,1,-1], [1,1,0], [-1, -1, 0] ]) #Need to code delta J = -1 as we don't know which ladder is the lowest Ka
dk_list = [0, 1]


#--------------------------------------------------------------------
# Define the quantum number range
#--------------------------------------------------------------------

nq_j = list(range(5, 16))
nq_ka = list(range(0, 6))

print('Range of J values:', nq_j)
print('Range of Ka values:', nq_ka, '\n')


#------------------------------------
# Input file and reading of the file
#------------------------------------
# Input file
if len(sys.argv) != 2:
    print('Usage: assignMatches.py matches')
    exit(1)

print('Careful: assignMatches.py needs 2 a-type DR matches and 2 b- or c-type DR matches for one transition connecting the two Ka ladders\n')

# The input file should contain 4 lines, one match per line:
# First the matches in the a ladders,
# then the cross-ladder matches


match_name = sys.argv[1]
if ('matches' in match_name):
    name_short = match_name.split('_')[1]



#--------------------------------------------------------------------
# Read the matches
#--------------------------------------------------------------------

with open(match_name, 'r') as match_file:
    match_lines = match_file.read().splitlines()

a_lad_1 = []
a_lad_1.append(float(match_lines[0].split()[0]))
a_lad_1.append(float(match_lines[0].split()[1]))
a_lad_1.sort()

a_lad_2 = []
a_lad_2.append(float(match_lines[1].split()[0]))
a_lad_2.append(float(match_lines[1].split()[1]))
a_lad_2.sort()

bc_lad_1 = []
bc_lad_1.append(float(match_lines[2].split()[0]))
bc_lad_1.append(float(match_lines[2].split()[1]))

bc_lad_2 = []
bc_lad_2.append(float(match_lines[3].split()[0]))
bc_lad_2.append(float(match_lines[3].split()[1]))

print('Matches in the first a-ladder:', a_lad_1)
print('Matches in the second a-ladder:', a_lad_2)
print('First cross-ladder match:', bc_lad_1)
print('Second  cross-ladder match:', bc_lad_2, '\n')
    
#--------------------------------------------------------------------
# label frequencies
#--------------------------------------------------------------------    

fa1 = a_lad_1[0]
fa2 = a_lad_1[1]
fa3 = a_lad_2[0]
fa4 = a_lad_2[1]

# Find the cross-ladder frequencies and the two a-ladders connected by it
fbc = set(bc_lad_1).intersection(bc_lad_2)
fbc = tuple(fbc)[0]
fbc_link = set(bc_lad_1)^set(bc_lad_2)

f_list = [fa1, fa2, fa3, fa4, fbc]

#--------------------------------------------------------------------
# Determine the initial rotational constants values
# and create a .par file
#--------------------------------------------------------------------    

# Assumptions:
# - spacing between the ladder = B +C
# - C = 0.8 B
# - A set so that the inertial defect is 0

ct_B = (fa2 - fa1)/1.8
ct_C = 0.8 * ct_B
ct_A = 1 / (1/ct_C - 1/ct_B)

print('Initially estimated rotational constants (in MHz):')
print('A = ', ct_A, '   B = ', ct_B, '   C = ', ct_C, '\n')

writePar(ct_A, ct_B, ct_C)
 

#--------------------------------------------------------------------
# Assign quantum numbers
#--------------------------------------------------------------------

# We start by assigning J, Ka, Kc to the lower level of the first a-ladder,
# and investigate all possible values for all other states
# There are 6 energy levels to determine

rms_list = []
index_list = []

counter = 0

#for crossLadder_rule in bc_rule: #Choose the b or c selection rule
with open('results_' + name_short, 'w') as out_file:    
    
    for crossLadder_rule in bc_rule:
        for i in range(len(nq_j)): # loop over J
            for j in range(len(nq_ka)): # loop over Ka       
                if (nq_j[i]-nq_ka[j] >= 0):  #Ka inferior or equal J
                    for dk in dk_list:  #Ka + Kc = J and J+1 cases      
                        # define the first Ka ladder
                        if (dk-nq_ka[j] <=0): # Kc has to be inferior or equal to J 
                            egl1 = np.array([nq_j[i], nq_ka[j], nq_j[i]-nq_ka[j]+dk])
                            egl2 = egl1 + a_rule
                            egl3 = egl2 + a_rule
                            egl_lad1 = np.array([egl1, egl2, egl3])
                        
                            #Identify the transition from the first ladder involved in the cross-ladder transitions
                            if (fa1 in fbc_link):
                                egl_a1 = np.array([egl1,egl2])
                            else:
                                egl_a1 = np.array([egl2,egl3])
                        
                            #---------------------------------------
                            # Define the second ladder possibilities 
                            #---------------------------------------
                    
                            for k in range(len(egl_a1)): 
                                egl4_list = []
                                egl5_list = []
                                egl6_list = []
                                lev1_list = [] #level from the first ladder involved in the cross-ladder
                                lev2_list = [] #level from the second ladder involved in the cross-ladder    
                                
                                        
                                # Based on which frequency of the second a-ladder is connected, there are 2 options for the possible energy levels connected:
                                # 4  and 5 if fa3 in fbc link, 5 and 6 otherwise
                                
                                #create a list of possible levels, 5 is always involved
                                egl5_1 = egl_a1[k] + crossLadder_rule
                                egl4_1 = egl5_1 - a_rule
                                egl6_1 = egl5_1 + a_rule
                                
                                
                                
                                # Only append the list if the level is physically possible
                                # i.e. Ka + Kc -J = 0 or 1 and Kc lower or equal J
                                # par = Ka + Kc -J
                                # pKc = Kc -J
                                par_4 = egl4_1[1] + egl4_1[2] - egl4_1[0]
                                par_5 = egl5_1[1] + egl5_1[2] - egl5_1[0]
                                par_6 = egl6_1[1] + egl6_1[2] - egl6_1[0]
                                pKc_4 = egl4_1[0] - egl4_1[2] 
                                pKc_5 = egl5_1[0] - egl5_1[2] 
                                pKc_6 = egl6_1[0] - egl6_1[2] 
                                
                                if (par_4 in dk_list and par_5 in dk_list and par_6 in dk_list and pKc_4 >= 0 and pKc_5 >= 0 and pKc_6 >= 0):
                                    egl4_list.append(egl4_1)
                                    egl5_list.append(egl5_1)
                                    egl6_list.append(egl6_1)
                                    lev1_list.append(egl_a1[k]) 
                                    lev2_list.append(egl_a1[k] + crossLadder_rule)
                                
                                
                                if (fa3 in fbc_link):
                                    egl4_2 = egl_a1[k] + crossLadder_rule
                                    egl5_2 = egl4_2 + a_rule
                                    egl6_2 = egl5_2 + a_rule
                                    
                                else:
                                    egl6_2 = egl_a1[k] + crossLadder_rule
                                    egl5_2 = egl6_2 - a_rule
                                    egl4_2 = egl5_2 - a_rule

                                                    
                                par2_4 = egl4_2[1] + egl4_2[2] - egl4_2[0]
                                par2_5 = egl5_2[1] + egl5_2[2] - egl5_2[0]
                                par2_6 = egl6_2[1] + egl6_2[2] - egl6_2[0]
                                pKc2_4 = egl4_2[0] - egl4_2[2] 
                                pKc2_5 = egl5_2[0] - egl5_2[2] 
                                pKc2_6 = egl6_2[0] - egl6_2[2] 
                                
                                if (par2_4 in dk_list and par2_5 in dk_list and par2_6 in dk_list and pKc2_4 >= 0 and pKc2_5 >= 0 and pKc2_6 >= 0):    
                                    egl4_list.append(egl4_2)
                                    egl5_list.append(egl5_2)
                                    egl6_list.append(egl6_2)
                                    lev1_list.append(egl_a1[k]) 
                                    lev2_list.append(egl_a1[k] + crossLadder_rule)
                                    
                                    
                                                
                            # define the second ladder and run the fit
                                for k in range(len(egl5_list)):
                                    egl4 = egl4_list[k]
                                    egl5 = egl5_list[k]
                                    egl6 = egl6_list[k]
                                    lev1 = lev1_list[k]
                                    lev2 = lev2_list[k]
                                    
                                    egl_lad2 = np.array([egl4, egl5, egl6])
                                
                                    
                                    if (-1 not in egl_lad2):
                                        print('\n_________________________________________________________\n')
                                        print('First ladder: ', egl1, egl2, egl3)
                                        print('Second ladder: ', egl4, egl5, egl6)
                                        print('Cross ladder transition: ', lev1, lev2)
                                        print('Cross ladder rule', crossLadder_rule)
                                        print('_________________________________________________________\n')
                                        
                                        out_file.write('\n_________________________________________________________\n')
                                        out_file.write('First ladder: %2s %2s %2s\n' %(egl1, egl2, egl3))
                                        out_file.write('Second ladder: %2s %2s %2s\n' %(egl4, egl5, egl6))
                                        out_file.write('Cross ladder transition: %2s %2s\n' %(lev1, lev2))
                                        out_file.write('Cross ladder rule %2s\n' %(crossLadder_rule))
                                        out_file.write('_________________________________________________________\n\n')
                                        
                                        # First case, ladder 2 below
                                        out_file.write('First case, ladder 2 below\n')
                                        counter += 1
                                        out_file.write('counter: %2s\n' %(counter))
                                        file_name = 'file_' + str(counter) 
                                        writeLin(file_name, egl_lad1, egl_lad2, lev1, lev2, f_list)
                                        runFit(file_name, counter, out_file)
                                        
                                        # Second case, ladder 2 above
                                        out_file.write('Second case, ladder 2 above\n')
                                        counter += 1                     
                                        out_file.write('counter: %2s\n' %(counter))       
                                        file_name = 'file_' + str(counter) 
                                        writeLin(file_name, egl_lad1, egl_lad2, lev2, lev1, f_list)
                                        runFit(file_name, counter, out_file) 
                            
    
    
    
    print('\n\n_________________________________________________________')
    print('_________________________________________________________')
    print('FINAL RESULTS'.center(55))
    print('RMS:', rms_list)
    print('BEST FIT: number {}, RMS = {}' .format(index_list[rms_list.index(min(rms_list))], min(rms_list)))
    print('_________________________________________________________\n')                        


out_file.close()
cleanPathPartly()

#--------------------------------------------------------------------
# Creation of a figure
#--------------------------------------------------------------------
            
figure(1) 
scatter(index_list,rms_list, color='red')
yscale('log')
show(block=True)


#--------------------------------------------------------------------
# Export the rms list
#--------------------------------------------------------------------

with open('rms_' + name_short, 'w') as rms_file:  
    for i in range(len(index_list)):
        rms_file.write('%2s %2s\n' %(index_list[i], rms_list[i]))
        